@@ -15,6 +15,7 @@ class NevergradSearch(HyperOpt):
             "budget_size": 100,
             "num_workers": 10,
         },
+        maximize_objective: bool = False,
         fixed_params: Union[dict, None] = None,
         reload_path: Union[str, None] = None,
         reload_list: Union[list, None] = None,
@@ -27,6 +28,7 @@ class NevergradSearch(HyperOpt):
             integer,
             categorical,
             search_config,
+            maximize_objective,
             fixed_params,
             reload_path,
             reload_list,
@@ -34,31 +36,27 @@ class NevergradSearch(HyperOpt):
             verbose,
         )
         self.space = NevergradSpace(real, integer, categorical)
-        # Initialize the surrogate model/hyperparam config proposer
-        if self.search_config["optimizer"] == "CMA":
-            self.hyper_optimizer = ng.optimizers.CMA(
-                parametrization=self.space.dimensions,
-                budget=self.search_config["budget_size"],
-                num_workers=self.search_config["num_workers"],
-            )
-        elif self.search_config["optimizer"] == "NGOpt":
-            self.hyper_optimizer = ng.optimizers.NGOpt(
-                parametrization=self.space.dimensions,
-                budget=self.search_config["budget_size"],
-                num_workers=self.search_config["num_workers"],
-            )
-        else:
-            raise ValueError("Please provide valid nevergrad optimizer type.")
+        self.init_optimizer()
+        self.search_name = "Nevergrad Wrapper Search"
 
         # Add start-up message printing the search space
         if self.verbose:
-            self.print_hello("Nevergrad Wrapper Search")
+            self.print_hello()
+
+    def init_optimizer(self):
+        """Initialize the surrogate model/hyperparam config proposer."""
+        assert self.search_config["optimizer"] in list(dict(ng.optimizers.registry).keys())
+        self.hyper_optimizer = ng.optimizers.registry[self.search_config["optimizer"]](
+                parametrization=self.space.dimensions,
+                budget=self.search_config["budget_size"],
+                num_workers=self.search_config["num_workers"],
+            )
 
     def ask_search(self, batch_size: int):
         """Get proposals to eval next (in batches) - Random Sampling."""
         # Generate list of dictionaries with different hyperparams to evaluate
-        self.last_batch_params = [self.hyper_optimizer.ask() for i in range(batch_size)]
-        param_batch = [params.value[1] for params in self.last_batch_params]
+        last_batch_params = [self.hyper_optimizer.ask() for i in range(batch_size)]
+        param_batch = [params.value[1] for params in last_batch_params]
         return param_batch
 
     def tell_search(self, batch_proposals, perf_measures):
@@ -68,13 +66,26 @@ class NevergradSearch(HyperOpt):
             prop_conf = dict(prop)
             if self.fixed_params is not None:
                 for k in self.fixed_params.keys():
-                    del prop_conf[k]
-            for last_prop in self.last_batch_params:
-                if last_prop.value[1] == prop_conf:
-                    x = last_prop
-                    break
-            # Get performance for each objective - tell individually to optim
-            self.hyper_optimizer.tell(x, perf_measures[i])
+                    if k in prop_conf.keys():
+                        del prop_conf[k]
+            # Only update space if candidate is in bounds
+            if self.space.contains(prop_conf):
+                x = self.hyper_optimizer.parametrization.spawn_child(
+                    new_value=((), prop_conf)
+                )
+                # Get performance for each objective - Negate values for max
+                if not self.maximize_objective:
+                    self.hyper_optimizer.tell(x, perf_measures[i])
+                else:
+                    self.hyper_optimizer.tell(x, [-1 * p for p in perf_measures[i]])
+
+    def refine_space(self, real, integer, categorical):
+        """Update the nevergrad search space."""
+        self.space.update(real, integer, categorical)
+        # Reinitialize the optimizer and provide data from previous updates
+        self.init_optimizer()
+        for iter in self.log:
+            self.tell_search([iter["params"]], [iter["objective"]])
 
     def get_pareto_front(self):
         """Get the pareto-front of the optimizer."""
